import 'package:flutter/material.dart';
import 'package:wow_views/common/text_style.dart';
import 'package:wowit/message.dart';

class WowText extends Text {
  static final RegExp checkMessageId =
      RegExp(r'^([A-Z]+[A-Z0-9]*_){1}[A-Z0-9_]*[A-Z0-9]{1}$');

  /// Text bình thường
  ///
  /// Mặc định 1 dòng, canh lề trái, từ trái sang phải
  const WowText._(
    super.data, {
    super.key,
    super.maxLines = 1,
    super.style,
    super.textAlign = TextAlign.start,
    super.textDirection = TextDirection.ltr,
    super.softWrap = true,
  }) : super(
          overflow:
              maxLines != null ? TextOverflow.ellipsis : TextOverflow.visible,
        );

  /// Widget Hiển thị văn bản
  ///
  /// mặc định tối đa 1 dòng, trái sang phải, dư hiển thị ...
  ///
  /// [data] có thể là văn bản hoặc id văn bản trên server
  ///
  /// Kiểu mã văn bản: chữ cái in hoa, số, dấu gạch dưới
  ///```dart
  /// 'HELLO_WORLD' // => đúng
  /// 'LANGUAGE_1' // => đúng
  /// 'LANGUAGE_1_VN' // => đúng
  /// 'HELLO_' // => sai
  /// '_HELLO' // => sai
  /// '1_HELLO' // => sai
  /// 'HELLOWORLD' // => sai
  /// 'Home_Title' // => sai
  ///```
  ///
  /// Ví dụ
  ///
  ///```dart
  /// WowText('Hello World'); // => 'Hello world'
  /// WowText('HELLO_WORLD'); // => 'Hello world' với văn bản lưu trên server
  ///```
  factory WowText(
    String? data, {
    Key? key,
    int? maxLines = 1,
    TextStyle? style,
    TextAlign? textAlign = TextAlign.start,
    TextDirection? textDirection = TextDirection.ltr,
    bool? softWrap = true,
  }) {
    String text = data ?? '';
    if (data != null && checkMessageId.hasMatch(data)) {
      text = Message.getMessage(data);
    }
    return WowText._(
      text,
      key: key,
      maxLines: maxLines,
      style: style,
      textAlign: textAlign,
      textDirection: textDirection,
      softWrap: softWrap,
    );
  }

  /// Text hỗn hợp
  ///
  /// Mặc định 1 dòng, canh lề trái, từ trái sang phải
  const WowText.rich(
    super.textSpan, {
    super.key,
    super.maxLines = 1,
    super.style,
    super.textAlign = TextAlign.start,
    super.textDirection = TextDirection.ltr,
    super.softWrap = true,
  }) : super.rich(
          overflow:
              maxLines != null ? TextOverflow.ellipsis : TextOverflow.visible,
        );

  /// Tiêu đề
  ///
  /// Thường dùng cho AppBar
  ///
  /// [data] có thể là văn bản hoặc id văn bản trên server
  ///
  /// Kiểu mã văn bản: chữ cái in hoa, số, dấu gạch dưới
  ///```dart
  /// 'HELLO_WORLD' // => đúng
  /// 'LANGUAGE_1' // => đúng
  /// 'LANGUAGE_1_VN' // => đúng
  /// 'HELLO_' // => sai
  /// '_HELLO' // => sai
  /// '1_HELLO' // => sai
  /// 'HELLOWORLD' // => sai
  /// 'Home_Title' // => sai
  ///```
  ///
  /// Ví dụ
  ///
  ///```dart
  /// WowText('Hello World'); // => 'Hello world'
  /// WowText('HELLO_WORLD'); // => 'Hello world' với văn bản lưu trên server
  ///```
  factory WowText.title(
    String? data, {
    Key? key,
    int? maxLines = 1,
    Color? color,
    FontWeight? fontWeight,
    TextAlign? textAlign = TextAlign.start,
    bool? softWrap = true,
  }) =>
      WowText(
        data,
        key: key,
        style: WowTextStyle.title?.copyWith(
          color: color,
          fontWeight: fontWeight,
        ),
        textAlign: textAlign,
        maxLines: maxLines,
        softWrap: softWrap,
      );

  /// Nội dung mặc định, bình thường
  ///
  /// [data] có thể là văn bản hoặc id văn bản trên server
  ///
  /// Kiểu mã văn bản: chữ cái in hoa, số, dấu gạch dưới
  ///```dart
  /// 'HELLO_WORLD' // => đúng
  /// 'LANGUAGE_1' // => đúng
  /// 'LANGUAGE_1_VN' // => đúng
  /// 'HELLO_' // => sai
  /// '_HELLO' // => sai
  /// '1_HELLO' // => sai
  /// 'HELLOWORLD' // => sai
  /// 'Home_Title' // => sai
  ///```
  ///
  /// Ví dụ
  ///
  ///```dart
  /// WowText('Hello World'); // => 'Hello world'
  /// WowText('HELLO_WORLD'); // => 'Hello world' với văn bản lưu trên server
  ///```
  factory WowText.bodyText(
    String? data, {
    Key? key,
    int maxLines = 1,
    Color? color,
    FontWeight? fontWeight,
    TextDecoration? decoration,
    FontStyle? fontStyle,
    TextAlign? textAlign = TextAlign.start,
    bool? softWrap = true,
  }) =>
      WowText(
        data,
        style: WowTextStyle.bodyText?.copyWith(
          color: color,
          fontWeight: fontWeight,
          decoration: decoration,
          fontStyle: fontStyle,
        ),
        textAlign: textAlign,
        maxLines: maxLines,
        softWrap: softWrap,
      );

  /// Nội dung chú thích, nhạt hơn mặc định
  ///
  /// [data] có thể là văn bản hoặc id văn bản trên server
  ///
  /// Kiểu mã văn bản: chữ cái in hoa, số, dấu gạch dưới
  ///```dart
  /// 'HELLO_WORLD' // => đúng
  /// 'LANGUAGE_1' // => đúng
  /// 'LANGUAGE_1_VN' // => đúng
  /// 'HELLO_' // => sai
  /// '_HELLO' // => sai
  /// '1_HELLO' // => sai
  /// 'HELLOWORLD' // => sai
  /// 'Home_Title' // => sai
  ///```
  ///
  /// Ví dụ
  ///
  ///```dart
  /// WowText('Hello World'); // => 'Hello world'
  /// WowText('HELLO_WORLD'); // => 'Hello world' với văn bản lưu trên server
  ///```
  factory WowText.caption(
    String? data, {
    Key? key,
    int? maxLines = 1,
    Color? color,
    FontWeight? fontWeight,
    TextDecoration? decoration,
    FontStyle? fontStyle,
    TextAlign? textAlign = TextAlign.start,
    bool? softWrap = true,
  }) =>
      WowText(
        data,
        style: WowTextStyle.caption?.copyWith(
          color: color,
          fontWeight: fontWeight,
          decoration: decoration,
          fontStyle: fontStyle,
        ),
        textAlign: textAlign,
        maxLines: maxLines,
        softWrap: softWrap,
      );

  /// Tiêu đề đầu mục
  ///
  /// Thường dùng cho các đầu mục, tiêu đề danh sách, ...
  ///
  /// [data] có thể là văn bản hoặc id văn bản trên server
  ///
  /// Kiểu mã văn bản: chữ cái in hoa, số, dấu gạch dưới
  ///```dart
  /// 'HELLO_WORLD' // => đúng
  /// 'LANGUAGE_1' // => đúng
  /// 'LANGUAGE_1_VN' // => đúng
  /// 'HELLO_' // => sai
  /// '_HELLO' // => sai
  /// '1_HELLO' // => sai
  /// 'HELLOWORLD' // => sai
  /// 'Home_Title' // => sai
  ///```
  ///
  /// Ví dụ
  ///
  ///```dart
  /// WowText('Hello World'); // => 'Hello world'
  /// WowText('HELLO_WORLD'); // => 'Hello world' với văn bản lưu trên server
  ///```
  factory WowText.listTitle(
    String? data, {
    Key? key,
    int? maxLines = 1,
    Color? color,
    FontWeight? fontWeight,
    TextDecoration? decoration,
    FontStyle? fontStyle,
    TextAlign? textAlign = TextAlign.start,
    bool? softWrap = true,
  }) =>
      WowText(
        data,
        style: WowTextStyle.listTitle?.copyWith(
          color: color,
          fontWeight: fontWeight,
          decoration: decoration,
          fontStyle: fontStyle,
        ),
        textAlign: textAlign,
        maxLines: maxLines,
        softWrap: softWrap,
      );

  /// Nội dung trong các thành phần danh sách
  ///
  /// Hoặc dùng cho nội dung to hơn bình thường
  ///
  /// [data] có thể là văn bản hoặc id văn bản trên server
  ///
  /// Kiểu mã văn bản: chữ cái in hoa, số, dấu gạch dưới
  ///```dart
  /// 'HELLO_WORLD' // => đúng
  /// 'LANGUAGE_1' // => đúng
  /// 'LANGUAGE_1_VN' // => đúng
  /// 'HELLO_' // => sai
  /// '_HELLO' // => sai
  /// '1_HELLO' // => sai
  /// 'HELLOWORLD' // => sai
  /// 'Home_Title' // => sai
  ///```
  ///
  /// Ví dụ
  ///
  ///```dart
  /// WowText('Hello World'); // => 'Hello world'
  /// WowText('HELLO_WORLD'); // => 'Hello world' với văn bản lưu trên server
  ///```
  factory WowText.listItem(
    String? data, {
    Key? key,
    int? maxLines = 1,
    Color? color,
    FontWeight? fontWeight,
    TextDecoration? decoration,
    FontStyle? fontStyle,
    TextAlign? textAlign = TextAlign.start,
    bool? softWrap = true,
  }) =>
      WowText(
        data,
        style: WowTextStyle.listItem?.copyWith(
          color: color,
          fontWeight: fontWeight,
          decoration: decoration,
          fontStyle: fontStyle,
        ),
        textAlign: textAlign,
        maxLines: maxLines,
        softWrap: softWrap,
      );

  /// Nội dung trong các nút
  ///
  /// [data] có thể là văn bản hoặc id văn bản trên server
  ///
  /// Kiểu mã văn bản: chữ cái in hoa, số, dấu gạch dưới
  ///```dart
  /// 'HELLO_WORLD' // => đúng
  /// 'LANGUAGE_1' // => đúng
  /// 'LANGUAGE_1_VN' // => đúng
  /// 'HELLO_' // => sai
  /// '_HELLO' // => sai
  /// '1_HELLO' // => sai
  /// 'HELLOWORLD' // => sai
  /// 'Home_Title' // => sai
  ///```
  ///
  /// Ví dụ
  ///
  ///```dart
  /// WowText('Hello World'); // => 'Hello world'
  /// WowText('HELLO_WORLD'); // => 'Hello world' với văn bản lưu trên server
  ///```
  factory WowText.button(
    String? data, {
    Key? key,
    int? maxLines = 1,
    Color? color,
    FontWeight? fontWeight,
    TextDecoration? decoration,
    FontStyle? fontStyle,
    TextAlign? textAlign = TextAlign.start,
    bool? softWrap = true,
  }) =>
      WowText(
        data,
        style: WowTextStyle.button?.copyWith(
          color: color,
          fontWeight: fontWeight,
          decoration: decoration,
          fontStyle: fontStyle,
        ),
        textAlign: textAlign,
        maxLines: maxLines,
        softWrap: softWrap,
      );
}
