import 'dart:async';
import 'dart:ui';
import 'dart:typed_data';
import 'dart:ui' as ui;
import 'package:flutter/rendering.dart';
import 'package:flutter/material.dart';
import 'package:google_maps_flutter/google_maps_flutter.dart';
import 'package:wow_views/screens/google_map/setup_marker.dart';
import 'package:wowit/methods.dart';

Future<BitmapDescriptor> canvasMarker(
  SetUpMarker setUpMarker,
  String? text,
) async {
  PictureRecorder recorder = PictureRecorder();
  Canvas canvas = Canvas(recorder);
  RenderParagraph renderParagraph = RenderParagraph(
    TextSpan(
      text: text,
      style: TextStyle(
        fontSize: setUpMarker.fontSize,
        fontWeight: setUpMarker.fontWeight,
      ),
    ),
    textDirection: ui.TextDirection.ltr,
    maxLines: 1,
  );

  final paintB = Paint()
    ..color = Colors.transparent
    ..strokeWidth = 10
    ..style = PaintingStyle.fill;

  TextSpan span = TextSpan(
    style: TextStyle(
      color: setUpMarker.textColor,
      fontSize: setUpMarker.fontSize,
      fontWeight: setUpMarker.fontWeight,
      background: paintB,
    ),
    text: text,
  );

  TextPainter tp = TextPainter(
    text: span,
    textAlign: TextAlign.center,
    textDirection: TextDirection.ltr,
  );
  // độ dài ngắn nhất cần có để render text
  double widthText = renderParagraph.getMinIntrinsicWidth(10000).ceilToDouble();
  // độ cao ngắn nhất cần có để render text
  double heightText =
      renderParagraph.getMinIntrinsicHeight(widthText).ceilToDouble();

  tp.layout();
  // Thiết lập màu sắc, kiểu tô để vẽ
  var paintRect = Paint()
    ..color = setUpMarker.selected
        ? const Color(0xffe75c2d)
        : (setUpMarker.backGroundColor ?? Colors.white)
    ..strokeWidth = 10
    ..style = PaintingStyle.fill;

  // cách lề trên
  double paddingTop = setUpMarker.padding?.top ?? 0;
  // cách lề dưới
  double paddingBottom = setUpMarker.padding?.bottom ?? 0;
  // Cách lề trái
  double paddingLeft = setUpMarker.padding?.left ?? 0;
  // Cách lề phải
  double paddingRight = setUpMarker.padding?.right ?? 0;
  // bo tròn
  double circular = 15;

  Path path = Path();

  // Vẽ hình chữ nhật
  // Chiều cao hình chữ nhật
  double rectHeight = heightText + paddingTop + paddingBottom;
  // Chiều dài hình chữ nhật
  double rectWidth = widthText + paddingLeft + paddingRight;
  // Toạ độ cạnh trái của hình
  double leftRect = 0;
  // Toạ độ cạnh trên của hình
  double topRect = 0;
  // ignore: avoid_print
  print(topRect);
  Rect rect = Rect.fromLTWH(leftRect, topRect, rectWidth, rectHeight);
  path.addRRect(RRect.fromRectAndRadius(rect, Radius.circular(circular)));

  // Vẽ hình tam giác nhỏ ngược bên dưới
  // chiều cao
  double triaHeight = 20;
  // chiều dài
  double triaWidth = 20;
  double r = 1;
  path.moveTo(rect.bottomCenter.dx + triaWidth / 2, rect.bottomCenter.dy);
  path.relativeLineTo(-triaWidth / 2 * r, triaHeight * r);
  path.relativeQuadraticBezierTo(
      -triaWidth / 2 * (1 - r), triaHeight * (1 - r), -triaWidth * (1 - r), 0);
  path.relativeLineTo(-triaWidth / 2 * r, -triaHeight * r);

  canvas.drawPath(path, paintRect);

  // Vẽ chữ lên nền
  tp.paint(
    canvas,
    Offset(
      leftRect + paddingLeft,
      topRect + paddingTop,
    ),
  );

  // Tổng chiều dài ảnh
  int totalWidth = (widthText + paddingLeft + paddingRight).ceil();
  // Tổng chiều cao ảnh
  int totalHeight = (rectHeight + triaHeight).ceil();
  //paint cavans
  Picture p = recorder.endRecording();
  ByteData? pngBytes =
      await (await p.toImage(totalWidth, totalHeight)).toByteData(
    format: ImageByteFormat.png,
  );
  pngBytes ??= ByteData(0);
  Uint8List data = Uint8List.view(pngBytes.buffer);

  return BitmapDescriptor.fromBytes(data);
}

Future<BitmapDescriptor> canvasMarkerDuAn(
    SetUpMarker setUpMarker, String text, String soCan) async {
  PictureRecorder recorder = PictureRecorder();
  Canvas canvas = Canvas(recorder);
  RenderParagraph renderParagraph = RenderParagraph(
    TextSpan(
      text: '$text\n$soCan',
      style: TextStyle(
          fontSize: setUpMarker.fontSize, fontWeight: setUpMarker.fontWeight),
    ),
    textDirection: ui.TextDirection.ltr,
    maxLines: 1,
  );

  final paintB = Paint()
    ..color = Colors.transparent
    ..strokeWidth = 10
    ..style = PaintingStyle.fill;

  TextSpan span = TextSpan(
      style: TextStyle(
          color: setUpMarker.textColor,
          fontSize: setUpMarker.fontSize,
          fontWeight: setUpMarker.fontWeight,
          background: paintB),
      text: '$text\n$soCan');

  TextPainter tp = TextPainter(
    text: span,
    textAlign: TextAlign.center,
    textDirection: TextDirection.ltr,
  );

  double widthText = renderParagraph.getMinIntrinsicWidth(10000).ceilToDouble();

  double heightText =
      renderParagraph.getMinIntrinsicHeight(widthText).ceilToDouble();
  tp.layout();
  var paintRect = Paint()
    ..color = setUpMarker.selected
        ? const Color(0xffe75c2d)
        : (setUpMarker.backGroundColor ?? Colors.white)
    ..strokeWidth = 10
    ..style = PaintingStyle.fill;
  final markerWidth = -widthText / 2;
  const markerHeight = 200.0;
  const paddingTop = 5.0;
  const rectTextHeight = 80.0;
  const circular = 15.0;
  const paddingRect = circular * 1.75;
  final leftRect = widthText + markerWidth - paddingRect;
  final totalWidth = (widthText.toInt()) * 2 +
      Methods.toInt((markerWidth + paddingRect).toString());

  final totalWidthRect = totalWidth - leftRect - circular;

  Rect rect =
      Rect.fromLTWH(leftRect, paddingTop, totalWidthRect, rectTextHeight);
  double x = 20, y = 20, r = 1;

  Path path = Path()
    ..addRRect(RRect.fromRectAndRadius(rect, const Radius.circular(circular)))
    ..moveTo(rect.bottomCenter.dx + x / 2, rect.bottomCenter.dy)
    ..relativeLineTo(-x / 2 * r, y * r)
    ..relativeQuadraticBezierTo(-x / 2 * (1 - r), y * (1 - r), -x * (1 - r), 0)
    ..relativeLineTo(-x / 2 * r, -y * r);

  canvas.drawPath(path, paintRect);

  tp.paint(
      canvas,
      Offset((widthText + markerWidth - 5),
          (rectTextHeight - heightText - 15) / 2.0));

//paint cavans
  Picture p = recorder.endRecording();
  ByteData? pngBytes = await (await p.toImage(
          totalWidth, Methods.toInt(markerHeight.toString())))
      .toByteData(
    format: ImageByteFormat.png,
  );
  pngBytes ??= ByteData(0);
  Uint8List data = Uint8List.view(pngBytes.buffer);
  return BitmapDescriptor.fromBytes(data);
}
